@@ -2,15 +2,20 @@ import json
 from pathlib import Path
 from typing import Iterator, Optional
 
-from ape.api.accounts import AccountAPI, AccountContainerAPI, TransactionAPI
+from ape.api import AccountAPI, AccountContainerAPI, PluginConfig, TransactionAPI
 from ape.types import AddressType, MessageSignature, TransactionSignature
 from eth_account.messages import SignableMessage
+from hexbytes import HexBytes
 
 from ape_trezor.client import TrezorAccountClient
-from ape_trezor.exceptions import TrezorSigningError
+from ape_trezor.exceptions import TrezorAccountError, TrezorSigningError
 from ape_trezor.hdpath import HDPath
 
 
+class TrezorConfig(PluginConfig):
+    hd_path: str = "m/44'/60'/0'/0"
+
+
 class AccountContainer(AccountContainerAPI):
     @property
     def _account_files(self) -> Iterator[Path]:
@@ -27,7 +32,7 @@ class AccountContainer(AccountContainerAPI):
     @property
     def accounts(self) -> Iterator[AccountAPI]:
         for account_file in self._account_files:
-            yield TrezorAccount(account_file_path=account_file)  # type: ignore
+            yield TrezorAccount(account_file_path=account_file)
 
     def save_account(self, alias: str, address: str, hd_path: str):
         """
@@ -59,7 +64,7 @@ class TrezorAccount(AccountAPI):
         return self.network_manager.ethereum.decode_address(self.account_file["address"])
 
     @property
-    def hdpath(self) -> HDPath:
+    def hd_path(self) -> HDPath:
         raw_path = self.account_file["hdpath"]
         return HDPath(raw_path)
 
@@ -70,7 +75,8 @@ class TrezorAccount(AccountAPI):
     @property
     def client(self) -> TrezorAccountClient:
         if self.account_client is None:
-            self.account_client = TrezorAccountClient(self.address, self.hdpath)
+            self.account_client = TrezorAccountClient(self.address, self.hd_path)
+
         return self.account_client
 
     def sign_message(self, msg: SignableMessage) -> Optional[MessageSignature]:
@@ -79,7 +85,6 @@ class TrezorAccount(AccountAPI):
         if version == b"E":
             signed_msg = self.client.sign_personal_message(msg.body)
 
-        # TODO: Uncomment when Trezor has released the EIP 712 update
         # elif version == b"\x01":
         #     signed_msg = self.client.sign_typed_data(msg.header, msg.body)
 
@@ -88,12 +93,52 @@ class TrezorAccount(AccountAPI):
                 f"Unsupported message-signing specification, (version={version!r})"
             )
 
-        return MessageSignature(*signed_msg)  # type: ignore
+        return MessageSignature(*signed_msg)
 
-    def sign_transaction(self, txn: TransactionAPI) -> Optional[TransactionSignature]:
+    def sign_transaction(self, txn: TransactionAPI, **kwargs) -> Optional[TransactionAPI]:
         txn_data = txn.dict()
-        if "data" in txn_data and not txn_data["data"]:
-            del txn_data["data"]
 
-        v, r, s = self.client.sign_transaction(txn_data)
-        return TransactionSignature(v=v, r=r, s=s)  # type: ignore
+        if "type" not in txn_data and "gasPrice" in txn_data:
+            tx_type = "0x00"
+
+        else:
+            tx_type = txn_data.pop("type", "0x00")
+            if isinstance(tx_type, int):
+                tx_type = HexBytes(tx_type).hex()
+            elif isinstance(tx_type, bytes):
+                tx_type = HexBytes(tx_type).hex()
+
+        # NOTE: `trezorlib` expects empty bytes when no data.
+        data = txn_data.get("data") or b""
+        if isinstance(data, str):
+            txn_data["data"] = HexBytes(data)
+
+        # NOTE: When creating contracts, use `""` as `to=` field.
+        txn_data["to"] = txn_data.get("to") or ""
+
+        # NOTE: Chain ID is required
+        chain_id = txn_data.pop("chainId")
+        if not chain_id:
+            chain_id = self.provider.chain_id
+
+        txn_data["chain_id"] = chain_id
+
+        # 'from' field not needed
+        if "from" in txn_data:
+            del txn_data["from"]
+
+        txn_data["gas_limit"] = txn_data.pop("gas", 0)
+
+        if tx_type == "0x00":
+            txn_data["gas_price"] = txn_data.pop("gasPrice", 0)
+            v, r, s = self.client.sign_static_fee_transaction(**txn_data)
+        elif tx_type == "0x02":
+            txn_data["max_gas_fee"] = txn_data.pop("maxFeePerGas", 0)
+            txn_data["max_priority_fee"] = txn_data.pop("maxPriorityFeePerGas", 0)
+            txn_data["access_list"] = txn_data.pop("accessList", [])
+            v, r, s = self.client.sign_dynamic_fee_transaction(**txn_data)
+        else:
+            raise TrezorAccountError(f"Message type {tx_type} is not supported.")
+
+        txn.signature = TransactionSignature(v=v, r=r, s=s)
+        return txn
