@@ -1,5 +1,5 @@
 from ape.utils import cached_property
-from trezorlib.tools import Address, parse_path  # type: ignore
+from trezorlib.tools import Address, parse_path
 
 
 class HDPath:
@@ -19,6 +19,9 @@ class HDPath:
     def __str__(self):
         return self.path
 
+    def __repr__(self):
+        return f"<{self}>"
+
     @cached_property
     def address_n(self) -> Address:
         return parse_path(self.path)
